#!/usr/bin/perl
BEGIN{
    push @INC,"/usr/share/oss/lib/";
}

use strict;
use Net::LDAP;
use Net::LDAP::Entry;
use oss;
use oss_utils;

# Make LDAP Connection
my $oss = oss->new();
$oss->connect_ldap();
my $DEBUG               = 0;
if( $oss->get_school_config('SCHOOL_DEBUG') eq 'yes' )
{
  $DEBUG = 1;
}
my $MY_BASE		=$oss->{LDAP_BASE};
my $attributes		="aDN $oss->{BIND_DN}\nap $oss->{APW}\nbDN $oss->{LDAP_BASE}\n";
my $oid			=undef;
my @member		=();
my @admins		=();
my $web         	=0;
my $cn	         	='';
my $grouptype         	='';
my $description        	='';
my $templateuser       	='';

while(<STDIN>)
{
  # Clean up the line!
  chomp; s/^\s+//; s/\s+$//;

  my ( $key, $value ) = split / /,$_,2;

  if( $key eq "oid" )
  {
         $oid = $value;
  }
  elsif( $key eq "templateuser" )
  {
	if( $templateuser =~ /no/i )
	{
		$templateuser = 0;
	}
  }
  elsif( $key eq "writeuid" )
  {
         push @admins, $value;
  }
  elsif( $key eq "member" )
  {
         push @member, $value;
  }
  else
  {
	if( $key eq 'web' )
	{
	       $web = 1;
	}
	elsif( $key eq "cn" )
	{
	       $cn = $value;
	}
	elsif( $key eq "description" )
	{
	       $description = $value;
	}
	elsif( $key eq "grouptype" )
	{
	       $grouptype = $value;
	}
        $attributes .= "$key $value\n";
  }
}

if( $oid ne "" )
{
  $MY_BASE=$oss->get_school_base($oid) or die $oss->{ERROR}->{text};
  $attributes .= "sDN $MY_BASE\n"; 
}

foreach my $DN (@member)
{
   $attributes .= "mDN  $DN\n"; 
   $attributes .= "mUID ".get_name_of_dn($DN);
}
if( $attributes !~ /mDN uid=admin,ou=people,$oss->{LDAP_BASE}/ )
{
    $attributes .= "mDN uid=admin,ou=people,$oss->{LDAP_BASE}\n"; 
}
foreach my $DN (@admins)
{
    $attributes .= "writerdn $DN\n"; 
}
if( $DEBUG )
{
  open(OUT,">/tmp/add_group");
  print OUT $attributes;
  close OUT; 
}
my $ERR = cmd_pipe("/usr/sbin/add_group -",$attributes);
if( $ERR !~ /^OK (\S+)/ )
{
  print $ERR;
  exit 1;
}
my $dn = $1;

if($web)
{
   system("/etc/init.d/apache2 reload");
}

#Starting the plugins
my $TMP = `echo "$dn" | /usr/sbin/oss_get_group.pl text`;
chomp $TMP;
my $TMPFILE = write_tmp_file($TMP);
system("/usr/share/oss/plugins/plugin_handler.sh add_group $TMPFILE");


print $oss->replydn($dn);

if( $grouptype eq 'primary' && $templateuser )
{ #we create a template user for this group
	my @tdn = $oss->get_entries_dn('(&(grouptype=primary)(role=templates))');
	$TMP  = "uid t$cn\n";
	$TMP .= "sn $description Template\n";
	$TMP .= "primarygroup ".get_name_of_dn($dn)."\n";
	$TMP .= "role templates\n";
	$TMP .= "description default\n";
	$TMP .= "userpassword ".$oss->{APW}."\n";
	if( $oid )
	{
	  $TMP .= "oid $oid\n";
	}
	if( $tdn[0] )
	{
	  $TMP .= "group ".get_name_of_dn($tdn[0])."\n";
	}
	my $TMPFILE = write_tmp_file($TMP);
	system("cat $TMPFILE | /usr/sbin/oss_add_user.pl &> /dev/null");
	system("rm $TMPFILE");
}

$oss->destroy();
