#!/usr/bin/perl
BEGIN{
    push @INC,"/usr/share/oss/lib/";
}

use strict;
use Net::LDAP;
use Net::LDAP::Entry;
use oss;
use oss_utils;

# Make LDAP Connection
my $oss = oss->new();
if( $oss->connect_ldap('admin') ne 1 ) {
  die $oss->{ERROR}->{text};
}

while(my $param = shift)
{
  if( $param =~ /text/i ) { $oss->{XML}=0; }
}

my $MY_BASE             =$oss->{LDAP_BASE};
my $DEBUG               = 0;
if( $oss->get_school_config('SCHOOL_DEBUG') eq 'yes' )
{
  $DEBUG = 1;
}
my $attributes          ="aDN $oss->{BIND_DN}\nap $oss->{APW}\nbDN $oss->{LDAP_BASE}\n";
my $role		="";
my $oid			="";
my $sDN			="";
my $maildomain		="";
my $uid			="";
my $userpassword        ="";
my @mails		= ();
my @folders             = ();
my @mailbox             = ();
#TODO this must come from attributes
my $LANG                ='DE';
my $dn;

while(<STDIN>)
{
	# Clean up the line!
	chomp; s/^\s+//; s/\s+$//;

	my ( $key, $value ) = split / /,$_,2;

	if( $key eq "oid" )
	{
		$oid = $value;
	}
	elsif( $key eq "sDN" )
	{
		$sDN	     = $value;
		$attributes .= "$key $value\n";
	}
	elsif( $key eq "userpassword_repeated" )
	{
		next;
	}
	elsif( $key eq "userpassword" )
	{
		$userpassword = $value;
		$attributes  .= "$key $value\n";
	}
	elsif( $key eq "mailbox" )
	{
	push @mailbox, $value;
		 $attributes .= "$key $value\n";
	}
	elsif( $key eq "mail" )
	{
		push @mails, $value;
	}
	elsif( $key eq "uid" )
	{
		$uid = $value;
		$attributes .= "$key $value\n";
	}
	elsif( $key eq "role" )
	{
		if( $value =~ /(.*),templates/ )
		{
			$role = "templates";
			my $tmp = $oss->get_primary_group($1);
			$attributes .= "primarygroup ".get_name_of_dn($tmp)."\n";
			my @tdn = $oss->get_entries_dn('(&(objectClass=schoolGroup)(role=templates))');
			if( $tdn[0] )
			{
			        $attributes .= "group ".get_name_of_dn($tdn[0])."\n";
			}
		}
		else
		{
			$role = $value;
		}  
		$attributes .= "$key $role\n";
	}
	else
	{
		$attributes .= "$key $value\n";
	}
}

if( $oid ne "" )
{
	$MY_BASE    = $oss->get_school_base($oid) or die $oss->{ERROR}->{text};
	$maildomain = $oss->get_school_config('SCHOOL_DOMAIN',$MY_BASE);
	$attributes .= "sDN $MY_BASE\n"; 
}
elsif( $sDN ne "" )
{
	$maildomain = $oss->get_school_config('SCHOOL_DOMAIN',$sDN);
}
else
{
	$maildomain = $oss->get_school_config('SCHOOL_DOMAIN');
}

if( $role ne "templates" )
{
	if( $uid ne "" && !contains($uid,\@mails) )
	{
	    push @mails, $uid;
	}

	foreach( @mails )
	{
	    if( ! /\@/ )
	    {
		$attributes .= "mail $_\@$maildomain\n";
	    }
	}
}

my $ERR = cmd_pipe("/usr/sbin/add_user -",$attributes);

if( $DEBUG )
{
	open(OUT,">/tmp/add_user.$uid");
	print OUT $attributes;
	print OUT "\nERR::$ERR";
	close OUT;
}


if( $ERR !~ /^OK (\S+) (\d+) (\S+)/ )
{
	print STDERR $ERR;
	exit 1;
}
$uid = $1;
$dn  = $3;

#Starting the plugins
my $TMP = `echo "$dn" | /usr/sbin/oss_get_user.pl text`;
$TMP .= "userpassword $userpassword";
chomp $TMP;
my $TMPFILE = write_tmp_file($TMP);
system("/usr/share/oss/plugins/plugin_handler.sh add_user $TMPFILE");

print $oss->replydn($dn);
$oss->destroy();
