#!/usr/bin/perl
BEGIN{
    push @INC,"/usr/share/oss/lib/";
}

use strict;
use Data::Dumper;
use oss;
use oss_utils;

# If neccessary read the commandline options
my $oss = oss->new();
while(my $param = shift)
{
  if( $param =~ /text/i ) { $oss->{XML}=0; }
}
if( $oss->connect_ldap('admin') ne 1 )
{
  die $oss->{ERROR}->{text};
}
if( ! $oss->connect_imap() )
{
  print $oss->{ERROR}->{text};
  exit 1;
}

my $DEBUG               = 0;
if( $oss->get_school_config('SCHOOL_DEBUG') eq 'yes' )
{ 
  $DEBUG = 1;
}
# Global variable
my $mesg;
my $dn               = '';
my $cn               = '';
my $oid              = '';
my $filter           = '';
my @attributes       = ();
my $scope            = 'sub';

open(OUT,">/tmp/get_group") if ($DEBUG);

while(<STDIN>)
{
  print OUT if ($DEBUG);

  # Clean up the line!
  chomp; s/^\s+//; s/\s+$//;

  my ( $key, $value ) = split / /,$_,2;
  if( !$value )
  {
	chomp $key;
        if( $key =~ /^cn=.*/i )
        {
	   $dn = $key;
	}
  }
  elsif ( $key eq 'filter' )
  {
        $filter = $value;
  }
  elsif ( $key eq 'oid' )
  {
	$oid = $value;
  }
  elsif ( $key eq 'cn' )
  {
	$cn = $value;
	$filter .= '('.$key.'='.$value.')';
  }
  elsif ( $key eq 'attributes' )
  {
        @attributes = split / /,$value;
  }
  else
  {
	$filter .= '('.$key.'='.$value.')';
  }
}
close OUT if ($DEBUG);

my $base = $oss->{LDAP_BASE};
if( $oid ne '' && $oid ne  'none' )
{
  $base  = 'ou=group,'.$oss->get_school_base($oid);
  $scope = 'one';
  if( $cn ne '' )
  {
    $dn = $oss->get_group_dn($cn,$oid);
  }
}

if( defined $dn && $dn ne '' )
{
        $base   = $dn;
        $scope  = 'base';
        $filter = '(objectclass=schoolGroup)';
}
elsif ( $filter ne '' )
{
	$filter = '(&(objectclass=schoolGroup)'.$filter.')';
}
else
{
	$filter = '(objectclass=schoolGroup)';
}

if( scalar(@attributes) )
{
   $mesg = $oss->{LDAP}->search( base   => $base,
                          scope  => $scope,
                          filter => $filter,
                          attrs  => \@attributes
                        );
}
else
{
   push @attributes, 'quota', 'fquota' ;
   $mesg = $oss->{LDAP}->search( base   => $base,
                          scope  => $scope,
                          filter => $filter,
                          attrs  => $oss->{DEFAULT_GROUP_ATTRIBUTES}
                        );
}

if( ! $mesg )
{
        print STDERR "ERROR Could not search for the group\n";
        exit 1;
}
my $result = $mesg->as_struct;
foreach $dn ( keys %{$result} )
{
	if( contains( 'quota', \@attributes ) )
	{
	  my($q_val,$q_used) = $oss->get_quota_group($dn);
	  push @{$result->{$dn}->{quota}},"$q_val";
	  push @{$result->{$dn}->{quotaused}},"$q_used";
	}

	if( contains( 'fquota', \@attributes ) )
	{
	  my($q_val,$q_used) = $oss->get_fquota_group($dn);
	  push @{$result->{$dn}->{fquota}},"$q_val";
	  push @{$result->{$dn}->{fquotaused}},"$q_used";
	}
}
print $oss->reply($result);

