#!/usr/bin/perl
BEGIN{
    push @INC,"/usr/share/oss/lib/";
}

use strict;
use Data::Dumper;
use oss_utils;
use oss;

# If neccessary read the commandline options
my $oss = oss->new();
while(my $param = shift)
{
  if( $param =~ /text/i ) { $oss->{XML}=0; }
}
# Initialisierung
#
if( ! $oss->connect_ldap() )
{
  print $oss->{ERROR}->{text};
  exit 1;
}


my $DEBUG               = 0;
if( $oss->get_school_config('SCHOOL_DEBUG') eq 'yes' )
{ 
  $DEBUG = 1;
}
my $mesg;
my @dns              = ();
my $dn               = '';
my $uid              = '';
my $oid              = '';
my $sDN              = '';
my $filter           = '';
my @attributes       = ();
my $scope            = 'sub';

open(OUT,">/tmp/get_user") if ($DEBUG);
while(<STDIN>)
{
  print OUT if ($DEBUG);

  # Clean up the line!
  chomp; s/^\s+//; s/\s+$//;

  my ( $key, $value ) = split / /,$_,2;
  if( !$value )
  {
	chomp $key;
	if( $key =~ /^uid=.*/i )
	{
	  push @dns, $key;
	}  
  }
  elsif ( $key eq 'filter' )
  {
	$filter = $value;
  }
  elsif ( $key eq 'sDN' )
  {
	$sDN = $value;
  }
  elsif ( $key eq 'oid' )
  {
	$oid = $value;
  }
  elsif ( $key eq 'uid' )
  {
	$uid = $value;
	$filter .= '('.$key.'='.$value.')';
  }
  elsif ( $key eq 'attributes' )
  {
	@attributes = split / /,$value;
  }
  else
  {
	$filter .= '('.$key.'='.$value.')';
  }
}
#DEBUG
close OUT if ($DEBUG);


#-----------------------------------------------------------------------------
if( ! $oss->connect_imap() )
{
  print $oss->{ERROR}->{text};
  exit 1;
}

if( ! scalar(@attributes) )
{
  @attributes = @{$oss->{DEFAULT_USER_ATTRIBUTES}};
}

my $base = $oss->{LDAP_BASE};
if( ! scalar( @dns ) )
{

	if( $oid ne '' )
	{
	  $base  = $oss->get_school_base($oid);
	  $scope = 'one';
	  if( $uid ne '' )
	  {
	    $dn = $oss->get_user_dn($uid,$base);
	  }
	  $base  = 'ou=people,'.$base;
	}
	elsif( $sDN ne '' )
	{
	  $base  = $sDN;
	  $scope = 'one';
	  if( $uid ne '' )
	  {
	    $dn = $oss->get_user_dn($uid,$base);
	  }
	  $base  = 'ou=people,'.$base;
	}

	if( defined $dn && $dn ne '' )
	{
		$base   = $dn;
		$scope  = 'base';
		$filter = '(objectclass=SchoolAccount)';
	}
	elsif ( $filter ne '' ) 
	{
		$filter = '(&(objectclass=SchoolAccount)'.$filter.')';
	}
	else
	{
		$filter = '(objectclass=SchoolAccount)';
	}

	$mesg = $oss->{LDAP}->search( base   => $base,
				  scope  => $scope,
				  filter => $filter,
				  attrs  => [ 'dn' ] 
				);

	if( ! $mesg )
	{
		print "ERROR Could not search the user\n";
		exit 1;
	}
	my $result = $mesg->as_struct;
        foreach my $dn ( keys %$result )
	{
	   push @dns, $dn;
	}
}
my $out = '';
foreach my $dn (@dns)
{
    if( $oss->{XML} ) 
    {
      $out .= hash_to_xml($oss->get_user($dn,\@attributes));
    }
    else
    {
      $out .= hash_to_text($oss->get_user($dn,\@attributes));
    }

}
if( $oss->{XML} ) 
{
  print reply_xml($out);
}
else
{
  print $out;
}

$oss->destroy();
exit;
