#!/usr/bin/perl -w
# Copyright (c) 2001 SuSE GmbH Nuernberg, Germany.  All rights reserved.
# Copyright (c) 2005 Peter Varkoly Fuerth, Germany.  All rights reserved.
#
#
# $Id: oss_import_user_list.pl,v 1.12 2007/03/30 12:29:54 pv Exp $
#
BEGIN{
    push @INC,"/srv/www/admin/lib"
}

use strict;
use Net::LDAP;
use Net::IMAP;
use Utils qw(%ATTRIBUTES);
use Data::Dumper;
use Getopt::Long;
use Config::IniFiles;
use CGI;
use ManageSieve;

# Global variable
my $mailserver   = 'mailserver';
my %options      = ();
my $result       = "";
my $role         = "students";
my $full         = 0;
my $lang         = 'DE';
my $sessionID    = 0;
my $input        = '/tmp/userlist.txt';
my $output       = '/srv/www/admin/import.html';
my $domain       = `hostname -d`;
my $LOGDIR       = "/var/log/";
my $PIDFILE      = "/var/run/import_user.pid";
my $DEBUG        = 0;
my $mailenabled  = 0;
my $userpassword = 0;
my $reqpwdchange = 0;
my $alias        = 0;

   
my $q            = new CGI; 
my $admin_user   = `oss_get_uid admin`;
my $admin_group  = `oss_get_gid sysadmins`;
my $admin_home   = `oss_get_home admin`;

# Get Admin Password
my $ldappw  = `/usr/sbin/get_admin_pw`;

# Parsing the attributes
$result = GetOptions(\%options,
                        "help",
                        "full",
                        "debug",
                        "alias",
                        "reqpwdchange",
                        "admin=s",
                        "input=s",
                        "output=s",
                        "role=s",
                        "domain=s",
                        "lang=s",
                        "mailenabled=s",
                        "userpassword=s",
                        "sessionID=s"
                        );

if (!$result && ($#ARGV != -1))
{
        usage();
        exit 1;
}
if ( defined($options{'input'}) )
{
	$input = $options{'input'};
}
if ( defined($options{'output'}) )
{
	$output = $options{'output'};
}
if ( defined($options{'help'}) )
{
        usage();
        exit 0;
}
if ( defined($options{'role'}) )
{
	$role = $options{'role'};
}
if ( defined($options{'full'}) )
{
	$full = 1;
}
if ( defined($options{'debug'}) )
{
	$DEBUG = 1;
}
if ( defined($options{'alias'}) )
{
	$alias = 1;
}
if ( defined($options{'reqpwdchange'}) )
{
	$reqpwdchange = 1;
}
if ( defined($options{'mailenabled'}) )
{
	$mailenabled = $options{'mailenabled'};
}
if ( defined($options{'userpassword'}) )
{
	$userpassword = $options{'userpassword'};
}
if ( defined($options{'domain'}) )
{
	$domain = $options{'domain'};
}
if ( defined($options{'lang'}) )
{
	$lang = $options{'lang'};
}
if ( defined($options{'sessionID'}) )
{
	$sessionID = $options{'sessionID'};
}
if ( defined($options{'admin'}) )
{
	$admin_user = $options{'admin'};
	$admin_group = `oss_get_primary_gid  $admin_user`;
	$admin_home  = `oss_get_home $admin_user`;
	$admin_user  = `oss_get_uid  $admin_user`;

}


daemonize($LOGDIR,$PIDFILE,$DEBUG);

# Setup the LDAP-Connection
my ($basedn, $ldapserver, $ldapport) = Utils::parse_file("/etc/openldap/ldap.conf", "BASE", "HOST", "PORT");
if( !defined $ldapport || $ldapport eq '' )
{
  $ldapport = 389;
}
my $ubase      = 'ou=people,'.$basedn;
my $gbase      = 'ou=group,'.$basedn;
my $ldap       = Net::LDAP->new($ldapserver, port=>$ldapport, version => 3);
if( !$ldap )
{
  close_on_error("Can not connect LDAP-Server: $ldapserver on Port $ldapport\n");
}
my $bind_dn    = 'cn=Administrator,'.$basedn;
if( ! $ldap->bind($bind_dn, password=>$ldappw) )
{
  close_on_error("Can not bind to LDAP-Server as $bind_dn with $ldappw\n");
}

# Setup the IMAP-Connection
my $imap = new Net::IMAP($mailserver, Debug => 0);
if( !$imap )
{
  close_on_error("Can not connect IMAP-Server: $mailserver\n");
}
$result   = $imap->login('cyrus', $ldappw );

# Setup the messages
my $allmessages = new Config::IniFiles( -file => '/srv/www/admin/lib/imap_lang_'.$lang.'.ini' );
my $message     = {};
foreach my $section ('GLOBAL', 'CREATE_USER','IMPORT_USER')
{
	my @parameters = $allmessages->Parameters($section);
        foreach(@parameters)
	{
            my $value = $allmessages->val($section, $_);
            $message->{$_} = $value;
        }

}

#print Dumper($message);
# setup an $session hash
my $session = {};
   $session->{confParam}->{LDAPserver} = $ldapserver;
   $session->{confParam}->{LDAPport}   = $ldapport;
   $session->{confParam}->{bind_dn}    = $bind_dn;
   $session->{confParam}->{passwd}     = $ldappw;
   $session->{confParam}->{baseDN}     = $basedn;
   $session->{confParam}->{gbase}      = $gbase;
   $session->{confParam}->{ubase}      = $ubase;
   $session->{confParam}->{LANG}       = $lang;
   $session->{confParam}->{IMAPserver} = $mailserver;
   $session->{confParam}->{OBSERV_HOME}    = Utils::read_rcconfig("SCHOOL_TEACHER_OBSERV_HOME");
   $session->{confParam}->{USE_EGROUPWARE} = Utils::read_rcconfig("SCHOOL_USE_EGROUPWARE");
   $session->{confParam}->{USE_OX}         = Utils::read_rcconfig("SCHOOL_USE_OX");
   $session->{confParam}->{bin_path}   = "/usr/sbin/";
   $session->{sessionID}               = $sessionID;

my $confParam = $session->{confParam};

# Setup the header
my $HHTML = '<html><head><meta http-equiv="Refresh" content="5"><title>Import</title>'.
	     '<link rel="stylesheet" type="text/css" href="/stylesheet.css"></head>'."\n<body>".
               $q->start_table({-class=>'AdminBorder', -width=>'100%'}).$q->start_Tr().$q->start_td().
               '<center><a href="/admin-cgi//menu.pl?sessionID='.$sessionID.'&tab=user&stab=new">'.
	       '<b>'.$q->h1($message->{working}).'<b></a></center>'.
               $q->end_td().$q->end_Tr().$q->end_table();

my $EHTML = '<html><head><title>Import</title>'.
	     '<link rel="stylesheet" type="text/css" href="/stylesheet.css"></head>'."\n<body>".
               $q->start_table({-class=>'AdminBorder', -width=>'100%'}).$q->start_Tr().$q->start_td().
               '<center><a href="/admin-cgi//menu.pl?sessionID='.$sessionID.'&tab=user&stab=new">'.
	       '<b>'.$q->h1($message->{succ_completed}).'<b></a></center>'.
               $q->end_td().$q->end_Tr().$q->end_table();

my $HTML  = "";

# now we start
&Utils::initATTRIBUTES();

add_user();

##############################################################################
## Print the option usage
sub usage
{

        print "import_user_list.pl [<options>]\n";
        print "Options:\n";
        print "  --help         Print this help message\n";
        print "  --input        The import file.\n";
	print "                 Default: /tmp/userlist.txt \n";
        print "  --role         Role of the users to import: students|teachers|administration\n";
	print "                 Default: students \n";
        print "  --full         List is a full list\n";
	print "                 Default: no \n";
        print "  --debug        Run in debug mode, no daemonize\n";
        print "  --admin        The login of the user who makes the import\n";
	print "                 Default: admin \n";
        print "  --domain       The domain of the school\n";
	print "                 Default: the output of `hostname -d` \n";
        print "  --mailenabled  Default value for mailenabled\n";
        print "  --userpassword Default value for userpassword\n";
	print "                 Default: each new user gets its own random password\n";
        print "  --alias        If set, the new users gets the default alias if not already exists\n";
	print "                 Default: no \n";
        print "  --reqpwdchange If set, the new users must change its password by the first login\n";
	print "                 Default: no \n";
        print "  --lang         The language of the messages\n";
	print "                 Default: DE \n";
        print "  --sessionID    The sessionID of the web session started this script.\n";
}


sub add_user
{

    my $NEWLIST    = {};
    my @CLASSES    = ();
    my @GROUPS     = ();
    my @ALLUID     = ();
    my %ALLUSER    = ();
    my @AKTUID 	   = ();
    my $DOMAIN     = $domain;
    my $MYCLASSES  = "";
    my $PRIMERCLASS= "";
    my @lines	   = ();
    my $errs;
    my @def_folder = split(/ /,$message->{def_folder});

    # Variable to handle the file header
    my $read_attrs    = $ATTRIBUTES{'attr_int'};
    my @attr_ext      = ();
    my %attr_ext_name = ();
    my $sep           = "";
    my %header        = ();
    
    # Get the list of the classes
    @CLASSES = @{Utils::get_all_classes($session,$ldap)};
    @GROUPS  = @{Utils::get_all_group($session,$ldap)};

    # Get the list of the users
    my $result = $ldap->search (  # perform a search
                           base   => $confParam->{'ubase'},
                           scope  => "one",  
			   filter => "(&(role=$role)(uid=*))",
			   attrs  => ['uid','sn', 'givenname', 'birthday']
			  );  
    foreach my $entry ($result->all_entries)
    {
       my $i = $entry->get_value('uid'); 
       push @ALLUID , $i;
       my $sn = $entry->get_value('sn');
       my $givenname = $entry->get_value('givenname');
       my $birthday = $entry->get_value('birthday');
       $ALLUSER{"$sn-$givenname-$birthday"} = $i;
    } 
  
    # -- building file header attributes
    foreach my $attr (@{$read_attrs})
    {
	if ( $message->{$attr} )
	{
          push @attr_ext, $message->{$attr};
	  $attr_ext_name{$message->{$attr}} = $attr;
	}
	else
	{
          push @attr_ext, $attr;
	  $attr_ext_name{uc($attr)} = $attr;
	}
    }
    my $muster = "";
    foreach my $i (@attr_ext)
    {
      if( $i ne "")
      {
        $muster.="$i|";
      }
    }
    chomp $muster;

    #-- reading the file in a variable
    open (INPUT, "<$input")  || close_on_error( "$message->{'cant_open_file'}" );
    while ( <INPUT> )
    {
	chomp; s/\r//;
    	push @lines, $_;
    }
    close (INPUT);
    #-- empty file
    if(scalar(@lines) < 2)
    {
         close_on_error( $message->{'emtpy_file'} );	
    }    
    #-- reading and evaluating the header
    my $HEADER = uc(shift @lines);
    #-- removing white spaces
    $HEADER =~ s/\s+//;
    #-- determine the field separator
    $HEADER =~ /($muster)(.+?)($muster)/;
    if( defined $2 )
    {
       $sep = $2;	
    }
    else
    {
         close_on_error( $message->{'bad_header'} );	
    }
    #-- insert for output needed fields
    if( $HEADER !~ /$message->{'uid'}/i)
    {
	$HEADER = $HEADER.$sep.$message->{'uid'};
    }
    if( $HEADER !~ /$message->{'userpassword'}/i)
    {
	$HEADER = $HEADER.$sep.$message->{'userpassword'};
    }
    if( $HEADER !~ /$message->{'birthday'}/i)
    {
         close_on_error( $message->{'miss_some_values'}." : ".$message->{'birthday'} );	
    }
    my $counter = 0;
    foreach my $i (split /$sep/,$HEADER)
    {
      $header{$counter} = $attr_ext_name{$i};
      $counter++;
    }
    foreach my $cl (@CLASSES)
    {
       $NEWLIST->{$cl}->{'header'} = $HEADER;
    }
    # Only studenst will be sorted in class lists.
    if( $role ne 'students' )
    {
       $NEWLIST->{$role}->{'header'} = $HEADER;
    }

    # Now we begins du setup the html side
    open( OUT, ">$output");
    print OUT "$HHTML\n$HTML\n</body></html>";
    close( OUT );
    chmod 0644, $output;
    foreach my $act_line (@lines)
    {
        # Logging
        print "------$act_line------\n";
        my %USER = ();
        # Setup some standard values
        $USER{'preferredlanguage'}  = $confParam->{LANG};
        $USER{'role'}               = $role;
        $USER{'class'}              = [];
        $USER{'group'}              = [];
	if( $mailenabled )
	{
	  $USER{'mailenabled'} = $mailenabled;
	}
	if( $userpassword )
	{
	  $USER{'userpassword'} = $userpassword;
	}
	if( $reqpwdchange )
	{
	  $USER{'reqpwdchange'} = 'yes';
	}
	if( $alias )
	{
	  $USER{'alias'} = 'yes';
	}
	my $uid   = '';
	my $ERROR = 0;
	my $ERRORS = '';

        # Pearsing the line
        my @line = split /$sep/, $act_line;
        # Continue if there was an empty line
        next if( scalar (@line) < 3); 
	foreach my $h (keys %header)
	{
	  if( $header{$h} eq "class" )
	  {
	     foreach my $c (split /\s+/,$line[$h])
	     {
	       push @{$USER{'class'}}, uc($c);
	     }
	  }
	  elsif( $header{$h} eq "group" )
	  {
	     foreach my $c (split /\s+/,$line[$h])
	     {
	       push @{$USER{'group'}}, uc($c);
	     }
	  }
	  else
	  {
	     if( ($header{$h} eq "uid" || $header{$h} eq "userpassword") && defined($line[$h]) )
	     {
		$line[$h] =~ s/\s//g;
	     }
	     $USER{$header{$h}}  = $line[$h]  || '';
	  }
    	}
	# It is more simplier :-)
	my @classes   = @{$USER{'class'}};  
        if( scalar(@classes) )
        {
          $MYCLASSES =join(' ',@classes);
          $PRIMERCLASS=$classes[0];
        }

	# If there is no domain defined we use the main mail domain
        if( !defined $USER{'domain'} )
	{
	    $USER{'domain'} = $DOMAIN;
        }

        #Check if all classes are present, someone who belongs to all classes
	#can not belong to not existend classes
        if( scalar(@classes) && $classes[0] !~ /all/i )
	{
	  foreach(@classes)
	  {
	    my $cn = uc($_);
	    if( !defined $NEWLIST->{$cn}->{'header'} )
	    {
	      my $command  = $confParam->{bin_path}."/oss_add_group.pl text ";
	      my $cmd_args = "cn $cn\n";
	      $cmd_args   .= "grouptype class\n";
	      $cmd_args   .= "description $message->{'class'} $cn\n";
	      $cmd_args   .= "bDN $confParam->{'baseDN'}\n";
	      $cmd_args   .= "mDN uid=admin,ou=people,$confParam->{'baseDN'}\n";
	      my $ret = Utils::cmd_pipe($command,$cmd_args);
	      # Logging
	      print "  ADD GROUP $cn: $ret\n";
	      push @CLASSES, $cn;
	      $NEWLIST->{$cn}->{'header'} = $HEADER;
	    }
	  }
	}

        #Check if all groups are present
	my @groups   = @{$USER{'group'}};  
	foreach(@groups)
	{
	  my $cn = uc($_);
	  next if( $cn =~ /^\-/ );
	  if( !contains($cn,\@GROUPS ))
	  {
	    my $command  = $confParam->{bin_path}."/oss_add_group.pl text ";
	    my $cmd_args = "cn $cn\n";
	    $cmd_args   .= "grouptype workgroup\n";
	    $cmd_args   .= "description $cn\n";
	    $cmd_args   .= "bDN $confParam->{'baseDN'}\n";
	    $cmd_args   .= "mDN uid=admin,ou=people,$confParam->{'baseDN'}\n";
	    my $ret = Utils::cmd_pipe($command,$cmd_args);
	    # Logging
	    print "  ADD GROUP $cn: $ret\n";
	    push @GROUPS, $cn;
	  }
	}

	# Analysing the birthday. We accept following forms: 
	# DDMMYYYY 
	# DD-MM-YYYY DD:MM:YYYY DD MM YYYY 
	# YYYY-MM-DD
	$USER{'birthday'} =~ tr/.: /---/;
	if( $USER{'birthday'} =~ /(\d{2})(\d{2})(\d{4})/)
	{
	  $USER{'birthday'} = "$3-$2-$1";
	}
	elsif ( $USER{'birthday'} =~ /(\d+)-(\d+)-(\d{4})/)
	{
	  $USER{'birthday'} = sprintf("$3-%02d-%02d",$2,$1);
        }
	elsif ( $USER{'birthday'} =~ /(\d{4})-(\d{2})-(\d{2})/)
	{
	  #Nothing to do it is all right.
	}
	else
	{
	   $ERRORS .= "<font color='red'> $USER{'givenname'} $USER{'sn'}: $message->{birthday_format_false}</font>\n";
	   $ERROR = 1;
	}

	# Do this user exist?
	if( exists($ALLUSER{"$USER{'sn'}-$USER{'givenname'}-$USER{'birthday'}"}) )
	{
	   $uid=$ALLUSER{"$USER{'sn'}-$USER{'givenname'}-$USER{'birthday'}"};
	   if( $USER{'uid'} eq "" )
	   {
	   	$USER{'uid'} = $uid;
	   }
	   else
	   {
	      if( $ALLUSER{"$USER{'sn'}-$USER{'givenname'}-$USER{'birthday'}"} ne $USER{'uid'} )
	      {
		$ERRORS .= "<font color='red'> $USER{'givenname'} $USER{'sn'} $USER{'birthday'}: $message->{same_person} $uid </font>\n";
	      }	
	   }
	}

	# And now let's do it 
	if( $uid ne '') {
	    # Logging
            print "  OLD USER $uid\n";
	    #First we make the older user
	    my $udn = 'uid='.$uid.','.$confParam->{'ubase'};
	    my $old_classes = Utils::get_all_classes($session,$ldap,$uid);
            $ERRORS .= "<b>$USER{'givenname'} $USER{'sn'}</b>: ".$message->{old_classes}.": ".join(" ",@$old_classes);
            $ERRORS .= " ".$message->{new_classes}.": ".$MYCLASSES."<br>\n";
            $result = $ldap->modify($udn, replace => { 'ou' =>$MYCLASSES });
            if( $PRIMERCLASS =~ /all/i )
	    {
		@classes = @CLASSES;
	    }
	    else
	    {
		@classes = @{$USER{'class'}};
	    }
	    my ($classes_to_del,$classes_to_add) = group_diff($old_classes,\@classes);
            foreach my $cn (@$classes_to_del)
	    {
		    my $gid = Utils::get_gid_of_group($session,$ldap,$cn);
                    $result = $ldap->modify($udn, delete => { OXGroupID => $gid } );
                    $result = $ldap->modify('cn='.$cn.','.$confParam->{'gbase'},delete => { member => $udn } );
	    	    $result = $ldap->modify('cn='.$cn.','.$confParam->{'gbase'},delete => { 'memberUID' =>"$uid" });
                    if($role eq 'students' && $confParam->{OBSERV_HOME} eq 'yes')
		    {
                       system("rm /home/classes/$cn/$uid");
                    }
		    if($role eq 'teachers' ) {
		       $result = $imap->deleteacl($cn,$uid);
		    }
            }
            foreach my $cn (@$classes_to_add)
	    {
		    my $gid = Utils::get_gid_of_group($session,$ldap,$cn);
                    $result = $ldap->modify($udn, add => { OXGroupID => $gid } );
                    $result = $ldap->modify('cn='.$cn.','.$confParam->{'gbase'},add => { member => $udn } );
	    	    $result = $ldap->modify('cn='.$cn.','.$confParam->{'gbase'},add => { 'memberUID' =>"$uid" });
                    if($role eq 'students' && $confParam->{OBSERV_HOME} eq 'yes')
		    {
                       system("ln -s /home/students/$uid  /home/classes/$cn/$uid");
                    }
		    if($role eq 'teachers' )
		    {
		       $result = $imap->setacl($cn,$uid,"lrswi");
		    }
            }
	    foreach( @{$USER{'group'}} )  
	    {
		my $cn = uc($_);
		if( $cn =~ s/^\-// )
		{
		    my $gid = Utils::get_gid_of_group($session,$ldap,$cn);
                    $result = $ldap->modify($udn, delete => { OXGroupID => $gid } );
                    $result = $ldap->modify('cn='.$cn.','.$confParam->{'gbase'},delete => { member => $udn } );
	    	    $result = $ldap->modify('cn='.$cn.','.$confParam->{'gbase'},delete => { 'memberUID' =>"$uid" });
		    if($role eq 'teachers' ) {
		       $result = $imap->deleteacl($cn,$uid);
		    }
		}
		else
		{
		    my $gid = Utils::get_gid_of_group($session,$ldap,$cn);
                    $result = $ldap->modify($udn, add => { OXGroupID => $gid } );
                    $result = $ldap->modify('cn='.$cn.','.$confParam->{'gbase'},add => { member => $udn } );
	    	    $result = $ldap->modify('cn='.$cn.','.$confParam->{'gbase'},add => { 'memberUID' =>"$uid" });
		    if($role eq 'teachers' )
		    {
		       $result = $imap->setacl($cn,$uid,"lrswi");
		    }
		}
	    }
	    push @AKTUID, $uid;
	}
	else
	{
	    # Loging
            print "  NEW USER\n";
	    # It is a new user 
	    if( !$USER{'userpassword'} || $USER{'userpassword'} eq "*")
	    {
	       for( my $i=0; $i < 8; $i++)
	       {
	         $USER{'userpassword'} .= pack( "C", int(rand(25)+97) );
	       } 
	        
	    }
	    elsif( $USER{'userpassword'} !~ /^[:\/\w\d§\$\%&!]+$/ )
	    {
                $ERRORS .= $message->{incorrect_passwd}.$USER{'userpassword'};
                $ERROR = 1;
            }
	    if($USER{"sn"} eq "" || $USER{"userpassword"} eq "" )
	    {
	        $ERRORS .= "<font color='red'> $USER{'givenname'} $USER{'sn'}: $message->{miss_some_values}</font>\n";
	        $ERROR = 1;
	    }
	    else
	    {
	        if(defined($USER{"uid"}) && ($USER{"uid"} =~ /[^a-zA-Z0-9-_\.]+/) )
		{  #   Match a non-word character
	            $ERRORS .= "<font color='red'> $USER{'givenname'} $USER{'sn'}: $message->{uid_invalid}</font>\n";
	            $ERROR = 1;
	        } 
	        $USER{"uid"} = lc($USER{"uid"});  # uid always lowercase

	        # Don't allow anybody or anyone as uid (these keywords are needed by cyrus for ACLs)
	        if( ($USER{"uid"} eq "anyone") || ($USER{"uid"} eq "anybody"))
		{ 
	            $ERRORS .= "<font color='red'> $USER{'givenname'} $USER{'sn'}: $message->{value_anyone_not_allowed}</font>\n";
	            $ERROR = 1;
	        } 

	        $USER{"mail"} = $USER{"uid"}."@".$USER{"domain"};

	        # at time only SMD5 is supported 
	        #if(length $USER{"userpassword"} < 5 || ( $USER{"pwmech"} eq "SMD5" ? 0 : length($USER{"userpassword"}) > 8 ) ) {
	        #    $ERRORS .= "<font color='red'> $USER{'givenname'} $USER{'sn'}: $message->{incorrect_passwd_length}</font>\n";
	        #    $ERROR = 1;
	        #}
	    }
	    if( !$ERROR )
	    { # Ist kein fehler aufgetreten wird der neue Benutzer angelegt.
	      $uid = to_ldap(\%USER);
              if( $uid =~ /uid: (\S+)/)
	      {
                $uid         = $1;
		$USER{'uid'} = $uid;
	        $ERRORS     .= "<b>$USER{'givenname'} $USER{'sn'}</b> $message->{created} Login: \"$uid\" $message->{class}: ".$MYCLASSES." <br>\n";
	        push @AKTUID, $uid;
              }
	      else
	      {
	        $ERROR   = 1;
		$ERRORS .= "<font color='red'> $USER{'givenname'} $USER{'sn'}: $message->{failed} $uid";
	      }
	    }
	}
	if( $ERROR eq 0 )
	{ # Prework for the list:
	    my $line = "";
	    foreach my $h (sort (keys %header))
	    {
	      if( $USER{$header{$h}} =~ /ARRAY/ )
	      {
	         $line .= join(" ",@{$USER{$header{$h}}}).$sep;
	      }
	      else
	      {
	         $line .= $USER{$header{$h}}.$sep;
              }
	    }
	    $line =~ s/$sep\$//;
            if( $role eq 'students' )
	    {
               $NEWLIST->{$PRIMERCLASS}->{$uid} = $line;
	    }
	    else
	    {
               $NEWLIST->{$role}->{$uid} = $line;
	    }
	}
        $HTML =  $q->start_table({-class=>'AdminBorder', -width=>'100%'}).$q->start_Tr().$q->start_td().
	         $ERRORS.$q->end_td().$q->end_Tr().$q->end_table()."\n".$HTML;
	open( OUT, ">$output");
	print OUT "$HHTML\n$HTML\n</body></html>";
	close( OUT );
        chmod 0644, $output;
    }
    # Logging
    print ">>>>>>>>>>>>>>>>DUMP OF THE NEW USER LIST<<<<<<<<<<<<<\n".Dumper(%$NEWLIST)."\n>>>>>>>>>>>>>>>END DUMP OF THE NEW USER LIST<<<<<<<<<<<<<<<<<<<";
    # Save the user list:
    my   $directory = `date +%Y-%m-%d.%H-%M`;
    chop $directory;
    system("mkdir -m 700 $admin_home/userimport.$directory");
    system("chown $admin_user:$admin_group $admin_home/userimport.$directory");
    if( $role eq 'students' )
    {
        foreach my $cl (@CLASSES)
	{
          my @ClassList = ($HEADER);
          foreach(keys %{$NEWLIST->{$cl}})
	  {
            if( $_ ne "header" )
	    {
            	push @ClassList, " ";
           	push @ClassList, $NEWLIST->{$cl}->{$_};
            }
          }
          if( scalar @ClassList > 1 )
	  {
                   save_file( \@ClassList, "$admin_home/userimport.$directory/userlist.$cl.txt" ); 
          }
        }
    }
    else
    {
        my @List = ($HEADER);
        foreach(keys %{$NEWLIST->{$role}})
	{
          if( $_ ne "header" )
	  {
          	push @List, " ";
         	push @List, $NEWLIST->{$role}->{$_};
          }
        }
        if( scalar @List > 1 )
	{
                save_file( \@List, "$admin_home/userimport.$directory/userlist.$role.txt" );
        }
    }
    # Delete old students
    if( $role eq 'students' &&  $full )
    {
	my $ind = {};
	$ind->{$_} = 1 foreach(@AKTUID);
	foreach my $uid (@ALLUID )
	{
	  if(not exists($ind->{$uid}))
	  {
		system("/usr/sbin/oss_delete_user $uid noarch");
                $HTML = $q->start_table({-class=>'AdminBorder', -width=>'100%'}).$q->start_Tr().$q->start_td().
	    	         "<b>Login: $uid</b> $message->{deleted} /home/archiv/$uid.tgz<br>".
			 $q->end_td().$q->end_Tr().$q->end_table()."\n".$HTML;
	        open( OUT, ">$output");
	        print OUT "$HHTML\n$HTML\n</body></html>";
	        close( OUT );
                chmod 0644, $output;
	  }		 
	}
    }
    # Starting archiving of the user in the background
    system("/usr/sbin/archiv_user &");
    # Syncing mysql settings if necessary
    if( $confParam->{USE_EGROUPWARE}  eq 'yes')
    {
	$HTML = $q->start_table({-class=>'AdminBorder', -width=>'100%'}).$q->start_Tr().$q->start_td().
	            '<b>'.$message->{syncingdb}.'<b>'.
	            $q->end_td().$q->end_Tr().$q->end_table()."\n".$HTML;
	open( OUT, ">$output");
	print OUT "$HHTML\n$HTML\n</body></html>";
	close( OUT );
        chmod 0644, $output;
       	system("ssh $mailserver /usr/sbin/oss_sync_group_ldap_mysql.pl");
       	system("ssh $mailserver /usr/sbin/oss_sync_user_ldap_mysql.pl");
    }
    system("/usr/sbin/archiv_user");
    system("rcnscd restart");
    open( OUT, ">$output");
    print OUT "$EHTML\n$HTML\n</body></html>";
    close( OUT );
    chmod 0644, $output;
    system("rm $PIDFILE");
    system("rm $input");
    $ldap->unbind();
    $imap->logout();
    exit;
}

sub to_ldap  ()
{
    my $params = shift;
    my @attrnames = ( "role", "uid", "sn", "givenname","birthday","alias","uidnumber",
                      "l", "preferredlanguage", "title", 'userpassword' ,"mailenabled","reqpwdchange",
                      "initials", "homephone", "mobile", "facsimiletelephonenumber", "pager",
                      "telephonenumber", "labeleduri", "street", "postalcode", "st", 'fquota', 'mquota' );

    my $errs        = "";
    my $classes     = $params->{'class'};
    my $groups      = $params->{'group'};
    # Set the access rights for the own LDAP attributes
    my $teacheraci  = "initials mail title l description street postalcode st c homephone mobile pager facsimiletelephonenumber telephonenumber labeleduri preferredLanguage jpegphoto";
    my $childaci    = "preferredLanguage";

    # Creata the command line
    my $command = $confParam->{bin_path}."/oss_add_user.pl text";

    my $cmd_args = "";

    # Setup LDAP attributes ou & the membeship in the classes
    my $ou="";
    if( scalar(@$classes) && @$classes[0] =~ /all/i)
    {
        $classes = Utils::get_all_classes($session,$ldap);
        $ou .= "all ";
        foreach my $i (@$classes)
	{
          $cmd_args .= "class $i\n";
        }
    }
    else
    {
       foreach my $i (@$classes)
       {
         $cmd_args .= "class $i\n";
         $ou .= "$i ";
       }
    }
    chop $ou;
    if( $ou ne "" )
    {
        $cmd_args .= "ou $ou\n";
    }

    foreach my $i (@$groups)
    {
      $cmd_args .= "group $i\n";
    }
    # Create description
    if($message->{$params->{'role'}} ne "" )
    {
        $cmd_args.= "description $message->{$params->{'role'}}\n";
    }

    # Set the access rights for the own LDAP attributes
    if( $params->{'role'} eq 'students' || $params->{'role'} eq 'templates' )
    {
      $ldap->modify($confParam->{'bind_dn'}, replace => {"defaultUserAci" => $childaci});
    }
    else
    {
      $ldap->modify($confParam->{'bind_dn'}, replace => {"defaultUserAci" => $teacheraci});
    }


    foreach my $attr (@attrnames)
    {
        if( defined $params->{$attr} && ($params->{$attr} ne "") )
	{
              $cmd_args .= "$attr ".$params->{$attr}."\n";
        }
    }

    $cmd_args .= "egroupware yes\n";
    $cmd_args .= "ox yes\n";

    foreach my $i (split(/ /,$message->{def_folder}))
    {
        $cmd_args .= "mailbox $i\n";
    }
    if(defined $params->{"admin"} && $params->{"admin"} eq "yes")
    {
        $cmd_args .= "admin yes\n";
    }
#print "$command $cmd_args\n";
    my $ret = Utils::cmd_pipe($command,$cmd_args);
    my $uid = get_name_of_dn($ret);
    if( $uid )
    {
        return "uid: $uid";
    }
    return $ret;
}

sub save_file($$){
   my $Lines = shift;
   my $File  = shift;

   open OUTPUT, ">$File";
   foreach(@$Lines){
	print OUTPUT $_."\n";
   }
   close OUTPUT;
   chown($admin_user,$admin_group,$File); 
}

######################################################################
#
# fork into background
#
sub daemonize
{
  my ($LOGDIR,$PIDFILE,$debug)=@_;
  system("rm $output");
  if ( ! $debug )
  {
    open STDIN,"/dev/null";
    open STDOUT,">>$LOGDIR"."/import_user.log";
    open STDERR,">>$LOGDIR"."/import_user.err";
    chdir "/";
    fork && exit 0;
    print STDERR "\n\n----------------------------------------\n";
    print STDERR `date`;
    print "\n\n----------------------------------------\n";
    print `date`;
    print time,": User import successfully forked into background and running on PID ",$$,"\n";
  }
  else
  { 
    print time,": User import running in debug-mode on PID ",$$,"\n";
  }
  open FILE,">$PIDFILE";
  print FILE $$;
  close FILE;
}

sub group_diff
{
    my $old   = shift;
    my $new   = shift;
    my @toadd = (); 
    my @todel = (); 
    foreach(@{$old})
    {
	if( !contains($_,$new))
	{
	   push @todel,$_;
	}
    }
    foreach(@{$new})
    {
	if( !contains($_,$old))
	{
	   push @toadd,$_;
	}
    }
    return ( \@todel, \@toadd );
}

sub contains
{
    my $a = shift;
    my $b = shift;
    foreach(@{$b})
    {
        if($a eq $_)
	{
                return 1;
        }
    }
    return 0;
}

sub close_on_error
{
    my $a = shift;
    print STDERR $a."\n";
    system("rm $PIDFILE");
    system("rm $input");
    exit 1;
}

sub get_name_of_dn($) {
    my $dn      = shift;

    my $first_tag = (split /,/,$dn)[0];
    if( defined $first_tag && $first_tag ne '' ) {
      return (split /=/,$first_tag)[1];
    }
    return undef;
}

1;
