=head1 NAME

 oss_utils

=head1 PREFACE

 This package contains some helper sripts.

=head1 SYNOPSIS

 #!/usr/bin/perl 
 
 BEGIN{ push @INC,"/usr/share/oss/lib/"; }
 
 use oss_utils;
 

=head1 DESCRIPTION


=over 2

=cut

require Exporter;
package oss_utils;

use strict;
use Socket;
use Time::Local;
use Digest::MD5  qw(md5_hex);
use MIME::Base64;
use Crypt::SmbHash;
use Storable qw(thaw freeze);
use vars qw(@ISA @EXPORT $VERSION);
@ISA = qw(Exporter);
@EXPORT = qw(
	&cmd_pipe
	&contains
	&get_name_of_dn
	&get_time_zones
	&get_parent_dn
	&getSocket
	&group_diff
	&hash_to_xml
	&hash_to_text
	&hash_password
	&parse_file
	&get_file
	&reply_xml
	&subscribe_folders
	&string_to_ascii
	&unsubscribe_folders
	&write_file
	&write_tmp_file
);
$VERSION = '0.01';
# Debug only
#use Data::Dumper;

# HASH to convert the utf8 -> ascii

my %to_ascii = (
		 'ü' => 'u',
		 'Ü' => 'U',
		 'ű' => 'u',
		 'Ű' => 'U',
		 'ú' => 'u',
		 'Ú' => 'U',
		 'ß' => 's',
		 'ő' => 'o',
		 'Ö' => 'O',
		 'Ő' => 'O',
		 'ö' => 'o',
		 'ä' => 'a',
		 'Ä' => 'A',
		 'á' => 'a',
		 'ă' => 'a',
		 'â' => 'a',
		 'è' => 'e',
		 'é' => 'e',
		 'í' => 'i',
		 'î' => 'i',
		 'ó' => 'o',
		 'ô' => 'o',
		 'Á' => 'A',
		 'Ă' => 'A',
		 'Â' => 'A',
		 'É' => 'E',
		 'Í' => 'I',
		 'Î' => 'I',
		 'Ó' => 'O',
		 'ç' => 'c',
		 'č' => 'c',
		 'ş' => 's',
		 'Ş' => 'S',
		 'ţ' => 't',
		 'Ţ' => 'T'
		 );

# Privat Variable
my $ldap_conf = '/etc/ldap.conf';
# SERVER
my $server_ip = "localhost";
my $server_port = "33333";
my $proto = 'tcp';
# UNIX
my $rendezvous = '/var/run/sessiond.sock';
my $readcount = 4096;
my $OXROOT    = '/var/www/html/openxchange';
# SSL
my $ssl_key_file = '/etc/ssl/servercerts/schoolserverkey.pem';
my $ssl_cert_file = '/etc/ssl/servercerts/schoolservercert.pem';
my $ssl_ca_file = '/etc/ssl/certs/YaST-CA.pem';
my $ssl_use_cert = 1; 
my $ssl_verify_mode = '0x01';
#######################################################################################
# End header                                                                          #
#######################################################################################


=item B<cmd_pipe($cmd,$arg)>

Pipes the arguments $arg tru the command $cmd

EXAMPLE:

  my $arg = "sn Varkoly
givenname Peter
role teacher
class 5A
class 7A
birthday 1967-04-17
";
   my $cmd = "/usr/sbin/add_user -";

   my $err = cmd_pipe($cmd,$arg); 

=cut 

sub cmd_pipe
{
        my $cmd = shift;
        my $arg = shift;
        my $err = "";
        my $pid = open(KID, "-|");

        if( defined( $pid)) {
            if ($pid) { # parent
                while(<KID>) {
                    chomp;
                    $err .= "$_ ";
                }
                close(KID);
            } else {    # child
                if( !open(CMD, "| $cmd 2>&1")) {
                            print STDERR "can not open $cmd cmd-pipe";
                        } else {
                            print CMD "$arg";
                            close(CMD);
                        }
              CORE::exit;
            }
        } else { # open failed
            $err = "cannot fork $cmd";
        }
        return "$err";
}
#-----------------------------------------------------------------------

=item B<contains($a,\@b)>

Returns true if the array @b contains the value $a;

=cut

sub contains {
    my $a = shift;
    my $b = shift;
    foreach(@{$b}){
        if($a eq $_) {
                return 1;
        }
    }
    return 0;
}

=item B<get_time_zones()>

Returns a hash with the timezones informations

EXAMPLE:

  my $timeZones = get_time_zones();

  print $timeZones->{default}."\n";
  foreach my $zone (@{$timeZones->{zones}}) {
    print "  $zone\n";
  }

=cut

sub get_time_zones() {
    my $tzf           =  '/usr/share/zoneinfo/zone.tab';
    my @timezones;
    my @dflt_timezone =  parse_file("/etc/sysconfig/clock", "TIMEZONE=");
    my $defaultTZ     =  $dflt_timezone[0];
    $defaultTZ        =~ s/\"//g;

    open(TZ, "$tzf") || return undef;
    while(<TZ>) {
        next if /^#/;
        next if /^\s*$/;
        m/(\w\w)\s+(.*?)\s+(.*?)\s+.*/;
        push @timezones, $3;
    }
    @timezones = sort(@timezones);
    #add the Standard 3 Letter timezones just to make sure
    push @timezones,( "ACT", "AET", "AGT", "ART", "AST", "BET", "BST", "CAT",
            "CET", "CNT", "CST", "CTT", "EAT", "ECT", "EET", "EST", "GMT",
            "HST", "IET", "IST", "JST", "MET", "MIT", "MST", "NET", "NST",
            "PLT", "PNT", "PRT", "PST", "SST", "UTC", "VST", "WET", "Etc/GMT",
            "Etc/GMT+1", "Etc/GMT+2", "Etc/GMT+3", "Etc/GMT+4", "Etc/GMT+5",
            "Etc/GMT+6", "Etc/GMT+7", "Etc/GMT+8", "Etc/GMT+9", "Etc/GMT+10",
            "Etc/GMT+11", "Etc/GMT+12", "Etc/GMT-1", "Etc/GMT-2", "Etc/GMT-3",
            "Etc/GMT-4", "Etc/GMT-5", "Etc/GMT-6", "Etc/GMT-7", "Etc/GMT-8",
            "Etc/GMT-9", "Etc/GMT-10", "Etc/GMT-11", "Etc/GMT-12",
            "Etc/GMT-13", "Etc/GMT-14", "Etc/UCT", "Etc/UTC");

   return { default=>$defaultTZ, zones=>\@timezones };

}
#-----------------------------------------------------------------------

=item B<get_name_of_dn(dn)>

Returns the last tag of an DN.

EXAMPLE:

my $uid = get_name_of_dn('uid=varkpete,ou=people,dc=extis,dc=de');


=cut

sub get_name_of_dn($) {
    my $dn      = shift;

    my $first_tag = (split /,/,$dn)[0];
    if( defined $first_tag && $first_tag ne '' ) {
      return (split /=/,$first_tag)[1];
    }
    return undef;
}
#-----------------------------------------------------------------------

=item B<get_parent_dn(dn)>

Returns the parent of an DN.

EXAMPLE:

my $base = get_parent_dn('uid=varkpete,ou=people,dc=extis,dc=de');


=cut

sub get_parent_dn($) {
    my $dn      = shift;

    my ($dummy, $parent) = split /,/,$dn,2;

    return $parent;
}
#-----------------------------------------------------------------------

=item B<group_diff(\@old_groups,\@new_groups)>

Returns the array pointers.

=cut

sub group_diff {
    my $old   = shift;
    my $new   = shift;
    my @toadd = ();
    my @todel = ();
    foreach(@{$old}) {
        if( !contains($_,$new)) {
           push @todel,$_;
        }
    }
    foreach(@{$new}) {
        if( !contains($_,$old)) {
           push @toadd,$_;
        }
    }
    return ( \@todel, \@toadd );
}
#-----------------------------------------------------------------------

# Subroutine to write a hash as an XML 
sub hash_to_xml{
    my $hash   = shift;
    my $xml    = "";
    my $ident  = '  ';

    foreach my $key ( keys %$hash ) {
      $xml .= "<dn dn=\"$key\">\n";
	foreach my $att ( keys %{$hash->{$key}} ) {
	    $ident  = '  ';
            foreach my $value (@{$hash->{$key}->{$att}}){
		  $xml .= "$ident<$att>$value</$att>\n";
	    }
	}
      $xml .= "</dn>\n";
    }
    return $xml;
}
#-----------------------------------------------------------------------

# Subroutine to write a hash as text 

sub hash_to_text{
    my $hash   = shift;
    my $text   = "";
    my $ident  = '  ';

    foreach my $key ( keys %$hash ) {
        $text .= $key."\n";
	foreach my $att ( keys %{$hash->{$key}} ) {
            foreach my $value (@{$hash->{$key}->{$att}}){
	        $text .= "$att $value\n";
	    }
	}
	$text .= "\n";
    }
    return $text;
}
#-----------------------------------------------------------------------

sub getSocket {

    my ($mode) = @_;
    my $sock;

    if( $mode == 1) {
        # unix socket

        socket($sock, PF_UNIX, SOCK_STREAM,0);
        connect($sock, sockaddr_un($rendezvous));
    } elsif ($mode == 2) {
        # ssl socket
        #$IO::Socket::SSL::DEBUG=1;

        $sock = IO::Socket::SSL->new( PeerAddr => $server_ip,
                                      PeerPort => $server_port,
                                      Proto    => $proto,
                                      SSL_key_file => $ssl_key_file,
                                      SSL_cert_file => $ssl_cert_file,
                                      SSL_ca_file => $ssl_ca_file,
                                      SSL_use_cert => $ssl_use_cert,
                                      SSL_verify_mode => $ssl_verify_mode );

    } elsif ($mode == 3) {
        # plain socket
        use IO::Socket;
        $sock = new IO::Socket::INET (
                                         PeerAddr => $server_ip,
                                         PeerPort => $server_port,
                                         Proto => $proto,
                                         );
    }

    if( ! defined $sock ) {
        print STDERR "Can not build up Socket!\n";
        print STDERR "ERRNO=<$!> in getSocket. Can not connect do SessionD\n";
        exit;
    }

    #print STDERR "building socker -> ".$sock."\n";

    return $sock;
}

=item B<hash_password($mech, $password)>

Hash user password for LDAP users. This subroutine proviedes the next
hash types: crypt, md5, smd5, sha, ssha 

EXAMPLE:

  my $hash = hash_password('crypt','12345678');

=cut

sub hash_password {

    my ($mech, $password) = @_;
    $mech = lc($mech);
    if ($mech  eq "crypt" ) {
        my $salt =  pack("C2",(int(rand 26)+65),(int(rand 26)+65));
        $password = crypt $password,$salt;
        $password = "{crypt}".$password;
    }
    elsif ($mech eq "md5") {
        my $ctx = new Digest::MD5();
        $ctx->add($password);
        $password = "{md5}".encode_base64($ctx->digest, "");
    }
    elsif ($mech eq "smd5") {
        my $salt =  pack("C5",(int(rand 26)+65),
                              (int(rand 26)+65),
                              (int(rand 26)+65),
                              (int(rand 26)+65),
                              (int(rand 26)+65)
                        );
        my $ctx = new Digest::MD5();
        $ctx->add($password);
        $ctx->add($salt);
        $password = "{smd5}".encode_base64($ctx->digest.$salt, "");
    }
    elsif( $mech eq "sha") {
        $password = sha1($password);
        $password = "{sha}".encode_base64($password, "");
    }
    elsif( $mech eq "ssha") {
        my $salt =  pack("C5", (int(rand 26)+65),
                               (int(rand 26)+65),
                               (int(rand 26)+65),
                               (int(rand 26)+65),
                               (int(rand 26)+65)
                        );
        $password = sha1($password.$salt);
        $password = "{ssha}".encode_base64($password.$salt, "");
    }
    return $password;
}
#-----------------------------------------------------------------------

=item B<parse_file(filename,searchfor)>

Parse a file for some attributes.

EXAMPLE:

  ($LDAP_BASE, $LDAP_SERVER, $LDAP_PORT) = parse_file($ldap_conf, "BASE", "HOST", "PORT");

=cut

sub parse_file {
    my $file = shift;
    my @searchfor = @_;

    my @erg = ();
    my $found = 0;
    my @a = split( /\n/, get_file($file));

    for(my $i=0; $i<=$#searchfor; $i++) {
        foreach (@a) {
            if($_ =~ /^$searchfor[$i]/i) {
                $_ =~ s/$searchfor[$i]\s*(.*)/$1/i;
                chomp($_);
                push @erg, $_;
                $found = 1;
                last;
            }
        }
        if($found != 1) {
            push @erg, "";
        } else {
            $found = 0;
        }
    }
    return @erg;
}
#-----------------------------------------------------------------------

=item B<get_file(filename)>

Reads a file into a variable.

EXAMPLE:

  my $file = get_file('/etc/hosts');

=cut

sub get_file($) {
  my $file     = shift;
  my $content  = '';
  local *F;
  open F, $file || return undef;
  while( <F> )
  {
     $content .= $_;
  }
  return $content;
}

#-----------------------------------------------------------------------

=item B<subscribe_folders(Users DN,List of the folders)>

Subscribes a list of folders for a user. This will be done directly into
the filesystem and not thru IMAP protocol

EXAMPLE:

   subscribe_folders('uid=admin,ou=people,dc=schule,dc=de',[ 'sysadmins', 'teachers' ] );

=cut

sub subscribe_folders($$)
{
  my $dn      = shift;
  my $folders = shift;
  my $uid     = get_name_of_dn($dn);

  #searching for subscribed file
  $uid =~ /^(.)/;
  my $subfile = "/var/lib/imap/user/$1/$uid.sub";
  if( $1 =~ /[0-9]/ )
  {
    $subfile = "/var/lib/imap/user/q/$uid.sub";
  }

  my $subs = get_file($subfile);

  my @lsubs = split /\t\n/, $subs;
  
  foreach( @{$folders} )
  {
    if( ! contains( $_, \@lsubs ) )
    {
      $subs = "$_\t\n".$subs;
    }
  }
  write_file($subfile, $subs );

}

=item B<unsubscribe_folders(Users DN,List of the folders)>

Unsubscribes a list of folders for a user. This will be done directly into
the filesystem and not thru IMAP protocol

EXAMPLE:

   unsubscribe_folders('uid=admin,ou=people,dc=schule,dc=de',[ 'sysadmins', 'teachers' ] );

=cut

sub unsubscribe_folders($$)
{
  my $dn      = shift;
  my $folders = shift;
  my $uid     = get_name_of_dn($dn);

  #searching for subscribed file
  $uid =~ /^(.)/;
  my $subfile = "/var/lib/imap/user/$1/$uid.sub";
  if( $1 =~ /[0-9]/ )
  {
    $subfile = "/var/lib/imap/user/q/$uid.sub";
  }

  my $subs = get_file($subfile);

  foreach( @{$folders} )
  {
    $subs =~ s/$_\t\n//m;
  }
  write_file($subfile, $subs );
}

#-----------------------------------------------------------------------

=item B<write_file(filename,var)>

Writes a scalar varibale into a file.

EXAMPLE:

  write_file('/tmp/12345',$TMP);

=cut

sub write_file($$) {
  my $file = shift;
  my $out  = shift;
  local *F;
  open F, ">$file" || die "Couldn't open file '$file' for writing: $!; aborting";
  local $/ unless wantarray;
  print F $out;
  close F;
}

#-----------------------------------------------------------------------

#-----------------------------------------------------------------------

=item B<write_tmp_file(var)>

Writes a scalar varibale into a temporary file.

EXAMPLE:

  my $tmp = write_tmp_file($TMP);

=cut

sub write_tmp_file($) {
  my $out  = shift;

  my $file = `mktemp /tmp/ossXXXXXXXX`;
  chomp $file;

  local *F;
  open F, ">$file" || die "Couldn't open file '$file' for writing: $!; aborting";
  local $/ unless wantarray;
  print F $out;
  close F;

  return $file;
}

#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
sub reply_xml {
    my $xml = shift;
    my $out = '';

    $out    = "<reply>\n";
    foreach my $line ( split /\n/,$xml ) {
      $out .= "  ".$line."\n";  
    }
    $out .= "</reply>\n";
    return $out;
}

#-----------------------------------------------------------------------
sub string_to_ascii {
    my $string = shift;

    $string =~ tr/üűÜŰßőÖŐöÄäáăâèéíîóôúÁĂÂÉÍÎÓÚçčşţŞŢ/uuUUsoOOoAaaaaeeiioouAAAEIIOUccstST/;
    $string =~ s/[\W]//g;
    return  $string;
}


1;
