#!/usr/bin/perl  -w
BEGIN{
    push @INC,"/srv/www/admin/lib"
}

use strict;
use Data::Dumper;
use Net::IMAP;
use IMAPUtils;
use Utils;
use Net::LDAP;
use Net::LDAP::Entry;
use Config::IniFiles;

my $entry;

# Get Admin Password
my $ldappw = `/usr/sbin/get_admin_pw`;
# Get ldap base
my $ldapbase = `sed '/#/d' /etc/openldap/ldap.conf | gawk '/base|BASE/ {print \$2}'`;

system("sed -i 's/index serviceAccesControl sub/index role,serviceAccesControl sub/' /etc/openldap/slapd.conf");
# Make LDAP Connection
my $LDAP = Net::LDAP->new('localhost',port =>389 ,version => 3);
$LDAP->bind('cn=Administrator,'.$ldapbase, password=>$ldappw);

# Now we convert the sysadmins
my $mesg = $LDAP->search( base   => 'ou=people,'.$ldapbase,
                          filter => "(gidnumber=500)",
		          scope  => 'one'
		     );

foreach $entry ( $mesg->entries )
{
   next if( $entry->exists('role'));
   $entry->add( role => 'sysadmins' );
   $entry->update( $LDAP );
   print "Updated :".$entry->dn()."\n";
}

# Now we convert the students
$mesg = $LDAP->search( base   => 'ou=people,'.$ldapbase,
                          filter => "(gidnumber=501)",
		          scope  => 'one'
		     );
foreach $entry ( $mesg->entries )
{
   next if( $entry->exists('role'));
   $entry->add( role => 'students' );
   $entry->update( $LDAP );
   print "Updated :".$entry->dn()."\n";
}

# Now we convert the teachers
$mesg = $LDAP->search( base   => 'ou=people,'.$ldapbase,
                          filter => "(gidnumber=502)",
		          scope  => 'one'
		     );
foreach $entry ( $mesg->entries )
{
   next if( $entry->exists('role'));
   $entry->add( role => 'teachers' );
   $entry->update( $LDAP );
   print "Updated :".$entry->dn()."\n";
}

# Now we convert the workstations
$mesg = $LDAP->search( base   => 'ou=people,'.$ldapbase,
                          filter => "(gidnumber=503)",
		          scope  => 'one'
		     );
foreach $entry ( $mesg->entries )
{
   next if( $entry->exists('role'));
   $entry->add( role        => 'workstations' );
   $entry->add( objectclass => 'SchoolAccount' );
   $entry->update( $LDAP );
   print "Updated :".$entry->dn()."\n";
}

$mesg = $LDAP->search( base   => 'ou=DHCP,'.$ldapbase,
                          filter => "(objectClass=DHCPEntry)",
		          scope  => 'sub'
		     );
foreach $entry ( $mesg->entries )
{
   next if( $entry->exists('role'));
   $entry->add( objectclass => 'SchoolWorkstation' );
   $entry->update( $LDAP );
   print "Updated :".$entry->dn()."\n";
}

# Now we convert the administration
$mesg = $LDAP->search( base   => 'ou=people,'.$ldapbase,
                          filter => "(gidnumber=504)",
		          scope  => 'one'
		     );
foreach $entry ( $mesg->entries )
{
   next if( $entry->exists('role'));
   $entry->add( role => 'administration' );
   $entry->update( $LDAP );
   print "Updated :".$entry->dn()."\n";
}

# Now we convert the template user
$mesg = $LDAP->search( base   => 'ou=people,'.$ldapbase,
                       filter => "(gidnumber=506)",
		       scope  => 'one'
		     );
foreach $entry ( $mesg->entries )
{
   next if( $entry->exists('role'));
   my $uid = $entry->get_value('uid');
   # since 2.5 the template users are member of the primary group
   # for that they are templates
   if( $uid eq 'tstudents' )
   {
      $entry->add(   description => 'default');
      $entry->replace( gidnumber => '501' );
      $LDAP->modify( 'cn=students,ou=group,'.$ldapbase,
      		      add => { member => $entry->dn() }
		   );
   }
   if( $uid eq 'tteachers' )
   {
      $entry->add(   description => 'default');
      $entry->replace( gidnumber => '502' );
      $LDAP->modify( 'cn=tteachers,ou=group,'.$ldapbase,
      		      add => { member => $entry->dn() }
		   );
   }
   if( $uid eq 'tadministration' )
   {
      $entry->add(   description => 'default');
      $entry->replace( gidnumber => '504' );
      $LDAP->modify( 'cn=tadministration,ou=group,'.$ldapbase,
      		      add => { member => $entry->dn() }
		   );
   }
   $entry->add( objectClass => 'SchoolAccount' );
   $entry->add( role => 'templates' );
   $entry->update( $LDAP );
   $entry->delete( objectClass => [ 'SchoolUserTemplate' ] );
   $entry->update( $LDAP );
   print "Updated :".$entry->dn()."\n";
}

# Now we convert the workgroups groups
$mesg = $LDAP->search( base   => 'ou=group,'.$ldapbase,
                       filter => "(objectclass=SchoolGroup)",
		       scope  => 'one'
		     );
foreach $entry ( $mesg->entries )
{
   next if( $entry->exists('grouptype'));
   my $cn = $entry->get_value('cn');
   $entry->add( grouptype   => 'workgroup' );
   $entry->update( $LDAP );
   print "Updated :".$entry->dn()."\n";
}

# Now we convert the primary groups
$mesg = $LDAP->search( base   => 'ou=group,'.$ldapbase,
                       filter => "(objectclass=SchoolPrimaryGroup)",
                       scope  => 'one'
		     );
foreach $entry ( $mesg->entries )
{
   next if( $entry->exists('grouptype'));
   my $cn = $entry->get_value('cn');
   $entry->delete( objectclass => [ "SchoolPrimaryGroup" ] );
   $entry->add( objectclass => "schoolGroup" ,
		grouptype   => 'primary', 
		role        => $cn 
	      );
   $entry->update( $LDAP );
   print "Updated :".$entry->dn()."\n";
}

$mesg = $LDAP->modify( 'cn=workstations,ou=group,'.$ldapbase,
	     add => { objectclass => 'schoolGroup' ,
                      grouptype   => 'primary',
                      role        => 'workstations'
                   }
	      );
# Now we convert the classes
$mesg = $LDAP->search( base   => 'ou=group,'.$ldapbase,
                       filter => "(objectclass=SchoolClass)",
		       scope  => 'one'
		     );
foreach $entry ( $mesg->entries )
{
   next if( $entry->exists('grouptype'));
   my $cn = $entry->get_value('cn');
   $entry->delete( objectclass => [ "SchoolClass" ]);
   $entry->add( objectclass => "schoolGroup" ,
		grouptype   => 'class' 
	      );
   $entry->update( $LDAP );
   print "Updated :".$entry->dn()."\n";
}

open(CONF,"/etc/sysconfig/schoolserver");
my $desc  = "";
my $key   = "";
my $value = "";
my $type  = "";
my $RO    = "";
my $Path  = "Network/Schoolserver/Basis";
$entry = Net::LDAP::Entry->new;  
$entry->add(objectClass=>'organizationalUnit');
$entry->add(ou=>'sysconfig');
$entry->dn( 'ou=sysconfig,'.$ldapbase );
$mesg = $LDAP->add( $entry );
$entry = Net::LDAP::Entry->new;  

# Maildomain
$mesg = $LDAP->search( base   => 'uid=admin,ou=people,'.$ldapbase,
                       filter => "(objectClass=*)",
		       scope  => 'base',
		       attrs  => [ 'mailDomain' ]
		     );
$value = $mesg->entry(0)->get_value('mailDomain');
$entry = Net::LDAP::Entry->new;  
$entry->add(objectClass=>'schoolConfiguration');
$entry->add(configurationPath=>'Network/Schoolserver/Basis');
$entry->add(configurationValueType=>'string'); 
$entry->add(configurationValueRO=>'yes'); 
$entry->add(configurationKey=>'SCHOOL_DOMAIN');
$entry->add(configurationValue=>"$value"); 
$entry->add(description=>"DNS Domain of the school"); 
$entry->dn( 'configurationKey=SCHOOL_DOMAIN,ou=sysconfig,'.$ldapbase );
$mesg = $LDAP->add( $entry );

# Home base
$entry = Net::LDAP::Entry->new;  
$entry->add(objectClass=>'schoolConfiguration');
$entry->add(configurationPath=>'Network/Schoolserver/Basis');
$entry->add(configurationValueType=>'string'); 
$entry->add(configurationValueRO=>'yes'); 
$entry->add(configurationKey=>'SCHOOL_HOME_BASE');
$entry->add(configurationValue=>'/home'); 
$entry->add(description=>"Base of the school home directory"); 
$entry->dn( 'configurationKey=SCHOOL_HOME_BASE,ou=sysconfig,'.$ldapbase );
$mesg = $LDAP->add( $entry );

# TFK
$entry = Net::LDAP::Entry->new;  
$entry->add(objectClass=>'schoolConfiguration');
$entry->add(configurationPath=>'Network/Schoolserver/Basis');
$entry->add(configurationValueType=>'yesno'); 
$entry->add(configurationValueRO=>'yes'); 
$entry->add(configurationKey=>'SCHOOL_USE_TFK');
$entry->add(configurationValue=>'no'); 
$entry->add(description=>"The Open School Server uses the Time for Kids web filter programm."); 
$entry->dn( 'configurationKey=SCHOOL_USE_TFK,ou=sysconfig,'.$ldapbase );
$mesg = $LDAP->add( $entry );

# SCHOOL_USE_ITOOL
$entry = Net::LDAP::Entry->new;  
$entry->add(objectClass=>'schoolConfiguration');
$entry->add(configurationPath=>'Network/Schoolserver/Settings');
$entry->add(configurationValueType=>'yesno'); 
$entry->add(configurationValueRO=>'no'); 
$entry->add(configurationKey=>'SCHOOL_USE_ITOOL');
$entry->add(configurationValue=>'no'); 
$entry->add(description=>"The Open School Server uses the Time for Kids web filter programm."); 
$entry->dn( 'configurationKey=SCHOOL_USE_ITOOL,ou=sysconfig,'.$ldapbase );
$mesg = $LDAP->add( $entry );

while(<CONF>)
{
  if(/## Section (I+)\./)
  {
    if( $1 eq 'II' )
    {
      $Path  = "Network/Schoolserver/Settings";
    }
    elsif( $1 eq 'III' )
    {
      $Path  = "Network/Schoolserver/Backup";
    }
    next;
  }
  elsif( /^$/ )
  { # Empty line
     $entry = Net::LDAP::Entry->new;  
     $desc  = '';
     $entry->add(objectClass=>'schoolConfiguration');
  }
  elsif( /^## Type:\s+(\w+)\s+(\w+)/)
  { # Type & RO
     $entry->add(configurationValueType=>"$1"); 
     if( $2 eq 'readonly' )
     {
       $entry->add(configurationValueRO=>'yes'); 
     }
     else
     {
       $entry->add(configurationValueRO=>'no'); 
     }
  }
  elsif( /^## Type:\s+(\w+)/)
  { # Type & RO
     $entry->add(configurationValueType=>"$1"); 
     $entry->add(configurationValueRO=>'no'); 
  }   
  elsif( /^## Default:\s+(\w+)/)
  { # Default value
     $entry->add(configurationDefaultValue=>"$1"); 
  }
  elsif( /^# (.*$)/)
  { # Description
     $desc .= $1." ";
  }
  elsif( /^(\w+)="(.*)"/)
  { # Key & Value
     next if( $1 eq 'SCHOOL_SUPPORTED_IMPORT_FORMATS' );
     next if( $1 eq 'SCHOOL_SORTED_HOME' );
     next if( $1 eq 'SCHOOL_USE_REMBO' );

     $entry->add(configurationPath=>"$Path");
     $entry->add(configurationKey=>"$1");
     if( $2 ne "" )
     {
       $entry->add(configurationValue=>"$2"); 
     }
     if( $Path eq "Network/Schoolserver/Basis" || 
	 $1 eq 'SCHOOL_IS_DEMO'  ||
	 $1 eq 'SCHOOL_USE_EGROUPWARE' || 
	 $1 eq 'SCHOOL_USE_OX' 
     )
     {
       $entry->delete(configurationPath=>[]); 
       $entry->delete(configurationValueRO=>[]); 
       $entry->add(configurationValueRO=>'yes'); 
       $entry->add(configurationPath=>"Network/Schoolserver/Basis");
     }
     $entry->add(description=>"$desc"); 
     $entry->dn( 'configurationKey='.$1.',ou=sysconfig,'.$ldapbase );
     $mesg = $LDAP->add( $entry );
  }
}

# Now we read /etc/imap/globals.conf in LDAP too
my $global = new Config::IniFiles( -file => '/etc/imap/globals.conf' );
my %keymap    = (
   ImportFileFormat       => "IMPORT_FILE_FORMAT",
   EnableUserSpamFrontend => "ENABLE_USER_SPAM_FRONTEND",
   EnableSieveEditor      => "ENABLE_SIVE_EDITOR",
   MonitorResolveAddr     => "MONITOR_RESOLVE_ADDR",
   NewUserChangePassword  => "NEW_USER_CHANGE_PASSWORD",
   UserJpegPhotoMaxHeight => "USER_JPEG_PHOTO_MAX_HEIGHT",
   UserJpegPhotoMaxWidth  => "USER_JPEG_PHOTO_MAX_WIDTH",
   MonitorServices        => "MONITOR_SERVICES",
   UseCookie              => "USE_COOKIE",
   CheckClientIP          => "CHECK_CLIENT_IP",
   DefaultPasswordHash	  => "DEFAULT_PASSWORD_HASH"
);
foreach my $section ($global->Sections)
{
   next if( $section =~ /SESSIOND|SUAD/ );
   $Path = "Network/Schoolserver/".$section;
   foreach my $key ( $global->Parameters($section) )
   {
     $value = $global->val($section,$key);
     $entry = Net::LDAP::Entry->new;  
     $entry->add(objectClass=>'schoolConfiguration');
     $key   = 'SCHOOL_'.$keymap{$key};
     if( $value =~ /^true$|^false$/)
     {
       $type = 'yesno';
       $value =~ s/true/yes/;
       $value =~ s/false/no/;
     }
     elsif( $value =~ /^\d+$/ )
     {
       $type = 'integer';
     }
     else
     {
       $type = 'string';
     }
     if( $key eq 'SCHOOL_IMPORT_FILE_FORMAT' )
     {
       $entry->add(configurationAvailableValue=> ['CSV','WinSV','SiBank','Schild-NRW']);
     }
     if( $key eq "SCHOOL_DEFAULT_PASSWORD_HASH" )
     {
       $entry->add(configurationAvailableValue=> ['SMD5','CRYPT']);
     }
     $entry->add(configurationPath=>$Path);
     $entry->add(configurationValueType=>$type); 
     $entry->add(configurationValueRO=>'no'); 
     $entry->add(configurationKey=>"$key");
     $entry->add(configurationValue=>"$value"); 
     $entry->dn( 'configurationKey='.$key.',ou=sysconfig,'.$ldapbase );
     $mesg = $LDAP->add( $entry );
   }
}

# Now we fix the DHCP Configuration
# Prepare pxelinux enviroment for the imaging
$mesg = $LDAP->search( base   => 'ou=DHCP,'.$ldapbase,
                       filter => "(cn=Pool1)",
                       scope  => 'sub'
                     );

$entry = $mesg->entry(0);
#looking for the ip-pool
my $pool = $entry->get_value( 'dhcpRange' );
$pool =~ /dynamic-bootp (\S+) (\S+)/;
$pool = $1;

#looking for the next server entry
my $nextserver = "";
foreach( $entry->get_value( 'dhcpStatements' ) )
{
   if( /next-server/ )
   {
     $nextserver = $_;
   }
}
if( $nextserver ne "" )
{
  $entry->delete( dhcpStatements => [ $nextserver, "option option-150 \"(nd)/menu.lst\"", "filename \"pxegrub\""  ]);
}
else
{
  $entry->delete( dhcpStatements => [ "option option-150 \"(nd)/menu.lst\"", "filename \"pxegrub\""  ]);
}
$entry->update( $LDAP );
my $dn = $entry->dn;
$dn =~ /^cn=Pool1,(.*)/;
$dn = $1;

# now we make pxelinux boot
$mesg = $LDAP->modify( $dn, add => { dhcpStatements => [ "filename \"pxelinux.0\"" , $nextserver, "use-host-decl-names on" ] });

# Prepare Room-1 for  Annon DHCP range
# Now we fix the DHCP Configuration
$mesg = $LDAP->add( 'cn=Room-1,'.$dn,
                attr   =>
                [
                  objectClass         => [ 'top','dhcpOptions','dhcpGroup','SchoolRoom' ],
                  cn                => 'Room-1',
                  serviceAccesControl => [ 'DEFAULT all:DENY internet:DENY printing:DENY mailing:DENY samba:DENY' , '06:00 DEFAULT' ],
                  dhcpNetMask         => 24,
                  dhcpRange           => $pool,
                  description         => 'ANON_DHCP'
                ]
);
